# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import copy
from chainladder.utils.cupy import cp
from chainladder.utils.sparse import sp
import numpy as np

def _get_full_expectation(cdf_, ultimate_):
    """ Private method that builds full expectation"""
    xp = cp.get_array_module(ultimate_.values)
    o, d = ultimate_.shape[-2:]
    cdf = copy.deepcopy(cdf_)
    cdf.values = xp.repeat(cdf.values[..., 0:1, :], o, axis=2)
    cdf.odims = ultimate_.odims
    cdf.valuation_date = ultimate_.valuation_date
    full = copy.deepcopy(ultimate_)
    r = full.values / cdf.values
    full.values = xp.concatenate((r, full.values), -1)
    full.ddims = np.append(cdf_.ddims, '9999-Ult')
    full.ddims = np.array([int(item.split('-')[0]) for item in full.ddims])
    full.vdim = ultimate_.vdims
    return full

def _get_full_triangle(full_expectation_, triangle_):
    """ Private method that builds full triangle"""
    full = full_expectation_
    full = (triangle_ + full - full[full.valuation<=triangle_.valuation_date])
    full.vdims = triangle_.vdims
    return full

def _get_cdf(ldf_):
    """ Private method that computes CDFs"""
    cdf = copy.deepcopy(ldf_)
    xp = cp.get_array_module(cdf.values)
    cdf.values = xp.flip(xp.cumprod(xp.flip(cdf.values, -1), -1), -1)
    cdf.ddims = np.array(
        [item.replace(item[item.find("-")+1:], '9999')
         for item in cdf.ddims])
    return cdf

class Common:
    """ Class that contains common properties of a "fitted" Triangle. """
    @property
    def cdf_(self):
        if not hasattr(self, 'ldf_'):
            raise AttributeError(
                "'" + self.__class__.__name__ + "'" +
                " object has no attribute 'cdf_'")
        return _get_cdf(self.ldf_)

    @property
    def ibnr_(self):
        if not hasattr(self, 'ultimate_'):
            raise AttributeError(
                "'" + self.__class__.__name__ + "'" +
                " object has no attribute 'ibnr_'")
        ibnr =  self.ultimate_ - self.latest_diagonal
        ibnr.vdims = self.ultimate_.vdims
        return ibnr

    @property
    def full_expectation_(self):
        if not hasattr(self, 'ultimate_'):
            raise AttributeError("'" + self.__class__.__name__ + "'" +
                                 " object has no attribute 'full_expectation_'")
        return _get_full_expectation(self.cdf_, self.ultimate_)

    @property
    def full_triangle_(self):
        if not hasattr(self, 'ultimate_'):
            raise AttributeError("'" + self.__class__.__name__ + "'" +
                                 " object has no attribute 'full_triangle_'")
        if hasattr(self, 'X_'):
            return _get_full_triangle(self.full_expectation_, self.X_)
        else:
            return _get_full_triangle(self.full_expectation_, self)
